﻿using System;
using System.Data;
using VV.Wrappers.SqlServer.Core.Dapper.Uow.Interfaces;

namespace VV.Wrappers.SqlServer.Core.Dapper.Uow
{
    public class DapperUnitOfWork : IDapperUnitOfWork, IDisposable
    {
        private readonly IDbConnection _connection;
        readonly ITransactionBase _transactionBase;
        private IDbTransaction DbTransaction { get; set; }

        public DapperUnitOfWork(IDbConnection connection,
                                 ITransactionBase transactionBase)
        {
            _connection = connection;
            _transactionBase = transactionBase;
        }

        public DapperUnitOfWork BeginTransaction()
        {
            DbTransaction = _connection
                                .BeginTransaction();

            _transactionBase
                .AddTransaction(DbTransaction);

            return this;
        }

        public bool Commit()
        {
            if (DbTransaction == null)
                return false;

            DbTransaction
                .Commit();

            return true;
        }

        public void Rollback()
        {
            DbTransaction
                .Rollback();
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
            _connection.Close();
            DbTransaction = null;
        }
    }
}
