﻿using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.ChangeTracking;
using Microsoft.EntityFrameworkCore.Metadata.Builders;

using System;
using System.Linq;

using VV.Wrappers.SqlServer.Core.Base.Entities;

namespace VV.Wrappers.SqlServer.Core.EF.Extensions
{
    public static class ConfigPropetiesDefault
    {
        internal static ModelBuilder AddDefaultProperties(this ModelBuilder modelBuilder)
        {
            foreach (var entityType in modelBuilder.Model.GetEntityTypes())
            {
                if (entityType.FindProperty("Id") != null)
                {
                    modelBuilder.Entity(entityType.Name).Property<DateTime?>("UltimaAlteracao");
                    modelBuilder.Entity(entityType.Name).Property<bool>("Deletado");
                }
            }

            return modelBuilder;
        }

        internal static EntityTypeBuilder AddIsDeletedFilter<T>(this EntityTypeBuilder<T> entityTypeBuilder) where T : EntityBase
        {
            entityTypeBuilder.HasQueryFilter(c => !Microsoft.EntityFrameworkCore.EF.Property<bool>(c, "Deletado"));
            return entityTypeBuilder;
        }

        internal static void SaveDefaultPropertiesChanges(ChangeTracker changeTracker)
        {
            foreach (var entry in changeTracker.Entries()
                .Where(e => (e.State == EntityState.Added ||
                            e.State == EntityState.Modified) && e.Entity is EntityBase))
            {
                entry.Property("UltimaAlteracao").CurrentValue = DateTime.Now;
            }

            foreach (var entry in changeTracker.Entries()
                .Where(p => p.State == EntityState.Deleted
                            && p.Entity is EntityBase))
            {
                entry.Property("Deletado").CurrentValue = true;
                entry.State = EntityState.Modified;
            }
        }
    }
}
