﻿using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Storage;
using System;
using System.Diagnostics.CodeAnalysis;
using System.Threading.Tasks;

using VV.Wrappers.SqlServer.Core.EF.Repositories.Interfaces;

namespace VV.Wrappers.SqlServer.Core.EF.Uow
{
    [ExcludeFromCodeCoverage]
    public class EFUnitOfWork : IEFUnitOfWork, IDisposable
    {
        private IDbContextTransaction DbTransaction { get; set; }
        private readonly DbContext _context;

        public EFUnitOfWork(IEFContext efContext)
        {
            _context = efContext.Context();
        }

        public async Task<EFUnitOfWork> BeginTransactionAsync()
        {
            DbTransaction = await _context
                                    .Database
                                    .BeginTransactionAsync()
                                    .ConfigureAwait(false);

            return this;
        }
        
        public async Task<bool> CommitAsync()
        {
            if (DbTransaction == null)
                return false;

            await _context
                    .SaveChangesAsync()
                    .ConfigureAwait(false);

            await _context
                    .Database
                    .CurrentTransaction
                    .CommitAsync()
                    .ConfigureAwait(false);

            return true;
        }

        public async Task RollbackAsync()
        {
            await _context
                    .Database
                    .CurrentTransaction
                    .RollbackAsync()
                    .ConfigureAwait(false);
        }

        public async Task SaveChangesAsync()
        {
            await _context
                    .SaveChangesAsync()
                    .ConfigureAwait(false);
        }

        public bool CurrentTransaction()
        {
            bool current = false;

            if(_context.Database.CurrentTransaction == null)
                current = true;

            return current;
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
            DbTransaction = null;
            _context.Dispose();
        }
    }
}
